
"""
Methods that controls the cykei client.
Controls communication and initializes the MainWindow.
"""

import json
import sys
import logging

from pkg_resources import resource_filename

from PySide2.QtGui import QIcon
from PySide2.QtWidgets import QApplication
from os.path import expanduser

from .window import MainWindow


def main():
    """Initializes server and window"""
    print("Starting Client.")

    # Load configuration
    record_dir = expanduser("~") + "/cyckei"
    config = json.load(open(record_dir + "/config.json", "r"))
    config["path"] = record_dir

    # Start logging
    log_file = "{}/client.log".format(record_dir)
    logging.basicConfig(filename=log_file, level=config["verbosity"],
                        format='%(asctime)s %(message)s')
    logging.info("--- Client started.")

    app = QApplication(sys.argv)
    app.setStyle("fusion")
    app.setWindowIcon(QIcon(resource_filename(
            "cyckei.client",
            "res/cyckei.png")))

    window = MainWindow(config)
    window.show()

    return app.exec_()


if __name__ == "__main__":
    main()



import sys

from subprocess import Popen, DEVNULL

from PySide2.QtGui import QIcon
from PySide2.QtWidgets import QApplication, QAction, QMenu, QSystemTrayIcon,\
    QMessageBox
from pkg_resources import require, DistributionNotFound, resource_filename


def main():
    app = QApplication(sys.argv)
    app.setStyle("fusion")
    app.setQuitOnLastWindowClosed(False)
    # TODO: Hide Dock icon Mac

    # Create the tray
    tray = QSystemTrayIcon(QIcon(resource_filename("cyckei.server",
                                                   "res/bolt.png")))
    menu = QMenu()
    controls = []

    try:
        version = require("cyckei")[0].version
    except DistributionNotFound:
        version = "(unpackaged)"

    controls.append(QAction("Cyckei Server {}".format(version)))
    controls[-1].setDisabled(True)
    menu.addAction(controls[-1])

    menu.addSeparator()

    controls.append(QAction("Launch Client"))
    controls[-1].triggered.connect(client)
    menu.addAction(controls[-1])

    controls.append(QAction("Stop Server"))
    controls[-1].triggered.connect(stop)
    menu.addAction(controls[-1])

    # Add the menu to the tray
    tray.setContextMenu(menu)
    tray.show()

    app.exec_()


def post_message(text):
    msg = QMessageBox()
    msg.setText(text)
    msg.exec_()


def client():
    Popen(["cyckei-client"], stdout=DEVNULL)


def stop():
    msg = ("Are you sure you want to close Cyckei Server?\n\n"
           "This will stop any current cycles and "
           "release control of all channels.")
    confirmation = QMessageBox()
    confirmation.setText(msg)
    confirmation.setStandardButtons(QMessageBox.Yes | QMessageBox.No)

    response = confirmation.exec_()

    if response == QMessageBox.Yes:
        sys.exit()



"""Controls log tab, which displays logs as they are being recorded"""

from os import path
from glob import glob
import subprocess
from PySide2.QtWidgets import QVBoxLayout, QHBoxLayout, QLabel, QStyleOption,\
    QPushButton, QListWidget, QListWidgetItem, QWidget, QPlainTextEdit, QStyle
from PySide2.QtGui import QPainter
from PySide2.QtCore import Slot, QRunnable


class LogViewer(QWidget):
    """Object of log tab"""
    def __init__(self, config, threadpool):
        QWidget.__init__(self)
        self.path = config["path"] + "/tests"
        self.threadpool = threadpool

        self.file_list = QListWidget()
        self.file_list.itemClicked.connect(self.list_clicked)

        # Create overall layout
        columns = QHBoxLayout(self)
        columns.addWidget(self.file_list)
        edit_rows = QVBoxLayout()
        columns.addLayout(edit_rows)
        columns.setStretch(0, 1)
        columns.setStretch(1, 5)

        # create edit_rows
        self.title_bar = QLabel()
        self.title_bar.setText("Select file on left to view log.")
        edit_rows.addWidget(self.title_bar)

        self.editor = QPlainTextEdit()
        edit_rows.addWidget(self.editor)

        controls = QHBoxLayout()
        edit_rows.addLayout(controls)

        # create control buttons
        reload = QPushButton()
        reload.setText("Reload")
        reload.clicked.connect(self.reload)
        controls.addWidget(reload)

        open_folder = QPushButton()
        open_folder.setText("Open Folder")
        open_folder.clicked.connect(self.open_explorer)
        controls.addWidget(open_folder)

        self.reload()

    def reload(self):
        worker = Reload(self)
        self.threadpool.start(worker)

    def open_explorer(self):
        """Open logging folder in explorer"""
        subprocess.Popen(r'explorer /select,"{}"'.format(self.path))

    def list_clicked(self, item):
        """Display text of clicked file in text box"""
        self.editor.setPlainText(item.content)

    def paintEvent(self, event):
        style_option = QStyleOption()
        style_option.initFrom(self)
        painter = QPainter(self)
        style = self.style()
        style.drawPrimitive(QStyle.PE_Widget, style_option, painter, self)


class Log(QListWidgetItem):
    """Object of log, stores title and content of file for quick access"""
    def __init__(self, file):
        super().__init__()
        self.file = file
        self.setText(self.file)
        with open(file) as file:
            self.content = file.read()


class Reload(QRunnable):
    def __init__(self, widget):
        super(Reload, self).__init__()
        self.widget = widget

    @Slot()
    def run(self):
        self.widget.file_list.clear()
        self.widget.editor.clear()
        logs = []
        files = glob("{}/*".format(self.widget.path))
        for file in files:
            if path.isdir(file):
                pass
            else:
                logs.append(Log(file))

        for log in logs:
            self.widget.file_list.addItem(log)



import logging

from threading import Thread
from pkg_resources import resource_filename
from json import load
from os.path import expanduser, exists
from os import makedirs
from shutil import copy

from . import server
from . import applet


def main():
    """
        This is the main entrypoint and routine of Cyckei Vayu.
        Vayu aims to make cyckei more efficient in many aspects.
    """
    print("Sarting Server.")

    try:
        from AppKit import NSBundle
        info = NSBundle.mainBundle().infoDictionary()
        info["LSBackgroundOnly"] = "1"
    except ModuleNotFoundError:
        pass

    # Setup recording direcory if unavailable
    record_dir = expanduser("~") + "/cyckei"
    file_structure(record_dir)

    # Load configuration
    config = load(open(record_dir + "/config.json", "r"))
    config["path"] = record_dir

    # Start logging
    log_file = "{}/server.log".format(record_dir)
    logging.basicConfig(filename=log_file, level=config["verbosity"],
                        format='%(asctime)s %(message)s')
    logging.info("--- Server started.")

    server_thread = Thread(target=server.main, args=(config,), daemon=True)
    server_thread.start()

    return applet.main()


def file_structure(path):
    """Checks for existing folder structure and sets up if missing"""

    if not exists(path):
        makedirs(path)
    if not exists(path + "/tests"):
        makedirs(path + "/tests")
    if not exists(path + "/config.json"):
        copy(resource_filename("cyckei.server", "res/default.config.json"),
             path + "/config.json")
    if not exists(path + "/batch.txt"):
        copy(resource_filename("cyckei.server", "res/batch.txt"),
             path + "/batch.txt")
    if not exists(path + "/scripts"):
        makedirs(path + "/scripts")
        copy(resource_filename("cyckei.server", "res/example-script"),
             path + "/scripts/example")


if __name__ == "__main__":
    main()



from subprocess import Popen, DEVNULL
from pkg_resources import require, DistributionNotFound


def main():

    try:
        version = require("cyckei")[0].version
    except DistributionNotFound:
        version = "(unpackaged)"

    print("\nWelcome to Cyckei {}!".format(version))
    Popen(["cyckei-client"], stdout=DEVNULL)
    Popen(["cyckei-server"], stdout=DEVNULL)


if __name__ == "__main__":
    main()



import json
import logging

from PySide2.QtCore import QRunnable, Slot, Signal, QObject
from PySide2.QtWidgets import QMessageBox
from os.path import exists
from os import makedirs
from datetime import date
from pkg_resources import resource_filename

from .socket import Socket


def not_none(value):
    """Sets a None value to "None" string"""
    return "None" if value is None else str(value)


def send(json):
    """Sends json to server and updates status with response"""
    # TODO: Load info from config
    socket = Socket("tcp://localhost", 5556)
    resp = socket.send(json)["response"]
    logging.info(resp)
    socket.socket.close()
    return resp


def prepare_json(channel, function, scripts):
    """Sets the channel's json script to current values"""
    protocol = scripts.get_script_by_title(
        channel.attributes["script_title"]).content
    json_packet = json.load(open(
        resource_filename("cyckei.client", "res/defaultJSON.json")))

    json_packet["function"] = function
    json_packet["kwargs"]["channel"] = channel.attributes["channel"]
    json_packet["kwargs"]["meta"]["cellid"] = channel.attributes["id"]
    json_packet["kwargs"]["meta"]["comment"] = channel.attributes["comment"]
    json_packet["kwargs"]["meta"]["package"] = channel.attributes["package"]
    json_packet["kwargs"]["meta"]["cell_type"] = channel.attributes["type"]
    temp_path = (
        channel.attributes["record_folder"]
        + "/"
        + str(date.today())
    )
    if not exists(temp_path):
        makedirs(temp_path)
    json_packet["kwargs"]["meta"]["path"] = (
        temp_path
        + "/"
        + channel.attributes["path"]
    )
    json_packet["kwargs"]["meta"]["mass"] = channel.attributes["mass"]
    json_packet["kwargs"]["meta"]["protocol_name"]\
        = channel.attributes["script_title"]
    json_packet["kwargs"]["meta"]["requester"]\
        = channel.attributes["requestor"]
    json_packet["kwargs"]["meta"]["channel"] = channel.attributes["channel"]
    json_packet["kwargs"]["meta"]["protocol"] = protocol
    json_packet["kwargs"]["protocol"] = protocol

    return json_packet


class Signals(QObject):
    alert = Signal(object)
    status = Signal(object, object)


class Ping(QRunnable):
    def __init__(self):
        super(Ping, self).__init__()
        self.signals = Signals()

    @Slot()
    def run(self):
        # TODO: Load info from config
        socket = Socket("tcp://localhost", 5556)
        response = socket.ping()
        self.signals.alert.emit(response)
        socket.socket.close()
# TODO: Separate status and feedback


class UpdateStatus(QRunnable):
    """Update status shown below controls by contacting server"""
    def __init__(self, channel):
        super(UpdateStatus, self).__init__()
        self.channel = channel
        self.signals = Signals()

    @Slot()
    def run(self):
        # TODO: Utilize configuration
        socket = Socket("tcp://localhost", 5556)
        info_channel = socket.info_channel(
            self.channel.attributes["channel"])["response"]
        channel_status = socket.channel_status(
            self.channel.attributes["channel"])["response"]
        try:
            status = (channel_status
                      + " - " + not_none(info_channel["state"])
                      + " | C: " + not_none(info_channel["current"])
                      + ", V: " + not_none(info_channel["voltage"]))
        except TypeError:
            status = info_channel
        logging.debug("Updating channel {} with satus {}".format(
            self.channel.attributes["channel"], status))
        self.signals.status.emit(status, self.channel)


class AutoFill(QRunnable):
    """Fill log text with value derived from cell identification"""
    def __init__(self, channel):
        super(AutoFill, self).__init__()
        self.channel = channel

    @Slot()
    def run(self):
        if self.channel.elements[2].text():
            self.channel.elements[3].setText(
                "{}A.pyb".format(self.channel.elements[2].text()))


class Read(QRunnable):
    """Tell channel to Rest() long enough to get voltage reading on cell"""
    def __init__(self, channel):
        super(Read, self).__init__()
        self.channel = channel
        self.signals = Signals()

    @Slot()
    def run(self):
        socket = Socket("tcp://localhost", 5556)
        package = json.load(open(
            resource_filename("cyckei.client", "res/defaultJSON.json")))
        package["function"] = "start"
        package["kwargs"]["channel"] = self.channel.attributes["channel"]
        package["kwargs"]["meta"]["path"] = (
            self.channel.attributes["record_folder"]
            + "/{}.temp".format(self.channel.attributes["channel"])
        )
        package["kwargs"]["protocol"] = """Rest()"""
        socket.send(package)

        info_channel = socket.info_channel(
            self.channel.attributes["channel"])["response"]
        try:
            status = ("Voltage of cell: "
                      + not_none(info_channel["voltage"]))
        except Exception:
            status = "Could not read cell voltage."

        package["function"] = "stop"
        socket.send(package)

        socket.socket.close()
        self.signals.alert.emit(status)


class Control(QRunnable):
    """Update json and send "start" function to server"""
    def __init__(self, channel, command, scripts):
        super(Control, self).__init__()
        self.channel = channel
        self.command = command
        self.scripts = scripts
        self.signals = Signals()

    @Slot()
    def run(self):
        if self.command == "start":
            script_ok = Check(self.scripts.get_script_by_title(
                    self.channel.attributes["script_title"]).content).run()
            if script_ok is False:
                return

        response = send(prepare_json(self.channel, self.command, self.scripts))
        self.signals.status.emit(response, self.channel)


class Check(QRunnable):
    def __init__(self, protocol):
        super(Check, self).__init__()
        self.protocol = protocol

    @Slot()
    def run(self):
        """Initiates checking tests"""
        passed, msg = self.legal_test(self.protocol)
        if not passed:
            return self.end_false(msg)
        passed, msg = self.run_test(self.protocol)
        if not passed:
            return self.end_false(msg)
        return True

    def legal_test(self, protocol):
        """Checks if script only contains valid commands"""
        conditions = ["#",
                      "for",
                      "AdvanceCycle()",
                      "CCCharge(",
                      "CCDischarge(",
                      "CVCharge(",
                      "CVDischarge(",
                      "Rest(",
                      "Sleep("]

        for line in protocol.splitlines():
            line = line.replace(" ", "")
            line = line.replace("\t", "")

            valid = False
            for condition in conditions:
                if line.startswith(condition) or not line:
                    valid = True
                    break
            if not valid:
                return False, "Illegal command: \"" + line + "\"."

        return True, "Passed"

    def run_test(self, protocol):
        """Checks if server can load script successfully"""
        packet = self.prepare_json(protocol)
        response = send(packet)
        if response == "Passed":
            return True, "Passed"
        return (False,
                "Server failed to run script. Error: \"{}\".".format(response))

    def end_false(self, reason):
        """Show message box with error statement and return false"""
        msg = QMessageBox()
        msg.setIcon(QMessageBox.Warning)
        msg.setText("Failed!")
        msg.setInformativeText("Script did not pass the check.")
        msg.setWindowTitle("Check Failed")
        msg.setDetailedText(reason)
        msg.exec_()
        return False

    def prepare_json(self, protocol):
        """create json to send to server"""
        json_packet = json.load(open(
            resource_filename("cyckei.client", "res/defaultJSON.json")))

        json_packet["function"] = "test"
        json_packet["kwargs"]["protocol"] = protocol

        return json_packet
